/*
 * bgexec.c : execute a command in background.
 * 
 * Copyright (c) 2005 Jiro Nishiguchi. All rights reserved.
 * 
 * This program and the accompanying materials are made available under
 * the terms of the Common Public License v1.0 which accompanies
 * this distribution, and is available at cpl-v10.html.
 */

#define BGEXEC_VERSION_OPTION "--version"
#define BGEXEC_VERSION "bgexec version 0.1"
#define BGEXEC_HELP_OPTION "--help"
#define BGEXEC_COPYRIGHT "Copyright (c) 2005 Jiro Nishiguchi. All rights reserved."
#define BGEXEC_USAGE "\
Usage: bgexec [command] (to execute the command)\n\
  or   bgexec --help (show this help)\n\
  or   bgexec --version (print product version and exit)"

#include <stdio.h>
#include <windows.h>

BOOL Execcmd(LPTSTR command);

int main(int argc, char *argv[])
{
	char *command;
	BOOL bSuccess;
	if (argc != 2)
	{
		printf("Error: could not execute.\n");
		printf("%s\n", BGEXEC_USAGE);
		return 1;
	}
	command = argv[1];
	if (strcmp(command, BGEXEC_HELP_OPTION) == 0)
	{
		printf("%s\n", BGEXEC_VERSION);
		printf("%s\n", BGEXEC_USAGE);
		return 0;
	}
	if (strcmp(command, BGEXEC_VERSION_OPTION) == 0)
	{
		printf("%s\n", BGEXEC_VERSION);
		printf("%s\n", BGEXEC_COPYRIGHT);
		return 0;
	}
	bSuccess = Execcmd(command);
	if (bSuccess == FALSE)
	{
		printf("Error: could not execute the command \"%s\"\n", command);
		return 1;
	}
	return 0;
}

BOOL Execcmd(LPTSTR command)
{
	STARTUPINFO sInfo;
	PROCESS_INFORMATION pInfo;
	BOOL success;
	GetStartupInfo(&sInfo);
	success = CreateProcess(NULL, command, NULL, NULL, FALSE, DETACHED_PROCESS, NULL, NULL, &sInfo, &pInfo);
	if (success == FALSE) return FALSE;
	CloseHandle(pInfo.hProcess);
	CloseHandle(pInfo.hThread);
	return TRUE;
}
